@@ -1,7 +1,7 @@
-from __future__ import print_function
-from __future__ import unicode_literals
+from __future__ import print_function, unicode_literals
 
 import os
+import sys
 import textwrap
 import time
 from fnmatch import fnmatch
@@ -11,45 +11,55 @@ import pytest
 import six
 
 import tox
-from .config import parseconfig
-from .result import ResultLog
-from .session import main
-from .venv import VirtualEnv
+from tox import venv
+from tox.config import parseconfig
+from tox.result import ResultLog
+from tox.session import Session, main
+from tox.venv import CreationConfig, VirtualEnv, getdigest
+
+mark_dont_run_on_windows = pytest.mark.skipif(os.name == "nt", reason="non windows test")
+mark_dont_run_on_posix = pytest.mark.skipif(os.name == "posix", reason="non posix test")
 
 
 def pytest_configure():
-    if 'TOXENV' in os.environ:
-        del os.environ['TOXENV']
-    if 'HUDSON_URL' in os.environ:
-        del os.environ['HUDSON_URL']
+    if "TOXENV" in os.environ:
+        del os.environ["TOXENV"]
+    if "HUDSON_URL" in os.environ:
+        del os.environ["HUDSON_URL"]
 
 
 def pytest_addoption(parser):
-    parser.addoption("--no-network", action="store_true",
-                     dest="no_network",
-                     help="don't run tests requiring network")
+    parser.addoption(
+        "--no-network",
+        action="store_true",
+        dest="no_network",
+        help="don't run tests requiring network",
+    )
 
 
 def pytest_report_header():
-    return "tox comes from: {}".format(repr(tox.__file__))
+    return "tox comes from: {!r}".format(tox.__file__)
 
 
 @pytest.fixture
-def newconfig(request, tmpdir):
-    def newconfig(args, source=None, plugins=()):
+def work_in_clean_dir(tmpdir):
+    with tmpdir.as_cwd():
+        yield
+
+
+@pytest.fixture(name="newconfig")
+def create_new_config_file(tmpdir):
+    def create_new_config_file_(args, source=None, plugins=()):
         if source is None:
             source = args
             args = []
         s = textwrap.dedent(source)
         p = tmpdir.join("tox.ini")
         p.write(s)
-        old = tmpdir.chdir()
-        try:
+        with tmpdir.as_cwd():
             return parseconfig(args, plugins=plugins)
-        finally:
-            old.chdir()
 
-    return newconfig
+    return create_new_config_file_
 
 
 @pytest.fixture
@@ -59,10 +69,19 @@ def cmd(request, capfd, monkeypatch):
     request.addfinalizer(py.path.local().chdir)
 
     def run(*argv):
-        key = str(b'PYTHONPATH')
+        key = str(b"PYTHONPATH")
         python_paths = (i for i in (str(os.getcwd()), os.getenv(key)) if i)
         monkeypatch.setenv(key, os.pathsep.join(python_paths))
+
         with RunResult(capfd, argv) as result:
+            prev_run_command = Session.runcommand
+
+            def run_command(self):
+                result.session = self
+                return prev_run_command(self)
+
+            monkeypatch.setattr(Session, "runcommand", run_command)
+
             try:
                 main([str(x) for x in argv])
                 assert False  # this should always exist with SystemExit
@@ -83,11 +102,11 @@ class RunResult:
         self.duration = None
         self.out = None
         self.err = None
+        self.session = None
 
     def __enter__(self):
         self._start = time.time()
-        # noinspection PyProtectedMember
-        self._capfd._start()
+        self._capfd.readouterr()
         return self
 
     def __exit__(self, exc_type, exc_val, exc_tb):
@@ -99,8 +118,9 @@ class RunResult:
         return self.out.splitlines()
 
     def __repr__(self):
-        return 'RunResult(ret={}, args={}, out=\n{}\n, err=\n{})'.format(
-            self.ret, ' '.join(str(i) for i in self.args), self.out, self.err)
+        return "RunResult(ret={}, args={}, out=\n{}\n, err=\n{})".format(
+            self.ret, " ".join(str(i) for i in self.args), self.out, self.err
+        )
 
 
 class ReportExpectMock:
@@ -115,13 +135,13 @@ class ReportExpectMock:
     def __getattr__(self, name):
         if name[0] == "_":
             raise AttributeError(name)
-        elif name == 'verbosity':
+        elif name == "verbosity":
             # FIXME: special case for property on Reporter class, may it be generalized?
             return 0
 
-        def generic_report(*args, **kwargs):
+        def generic_report(*args, **_):
             self._calls.append((name,) + args)
-            print("%s" % (self._calls[-1],))
+            print("{}".format(self._calls[-1]))
 
         return generic_report
 
@@ -136,8 +156,10 @@ class ReportExpectMock:
                 return call
             newindex += 1
         raise LookupError(
-            "looking for %r, no reports found at >=%d in %r" %
-            (cat, self._index + 1, self._calls))
+            "looking for {!r}, no reports found at >={:d} in {!r}".format(
+                cat, self._index + 1, self._calls
+            )
+        )
 
     def expect(self, cat, messagepattern="*", invert=False):
         __tracebackhide__ = True
@@ -152,13 +174,16 @@ class ReportExpectMock:
                 lmsg = str(lmsg).replace("\n", " ")
                 if fnmatch(lmsg, messagepattern):
                     if invert:
-                        raise AssertionError("found %s(%r), didn't expect it" %
-                                             (cat, messagepattern))
+                        raise AssertionError(
+                            "found {}({!r}), didn't expect it".format(cat, messagepattern)
+                        )
                     return
         if not invert:
             raise AssertionError(
-                "looking for %s(%r), no reports found at >=%d in %r" %
-                (cat, messagepattern, self._index + 1, self._calls))
+                "looking for {}({!r}), no reports found at >={:d} in {!r}".format(
+                    cat, messagepattern, self._index + 1, self._calls
+                )
+            )
 
     def not_expect(self, cat, messagepattern="*"):
         return self.expect(cat, messagepattern, invert=True)
@@ -174,17 +199,16 @@ class pcallMock:
         self.stderr = stderr
         self.shell = shell
 
-    def communicate(self):
+    @staticmethod
+    def communicate():
         return "", ""
 
     def wait(self):
         pass
 
 
-@pytest.fixture
-def mocksession(request):
-    from tox.session import Session
-
+@pytest.fixture(name="mocksession")
+def create_mocksession(request):
     class MockSession(Session):
         def __init__(self):
             self._clearmocks()
@@ -203,9 +227,7 @@ def mocksession(request):
         def make_emptydir(self, path):
             pass
 
-        def popen(self, args, cwd, shell=None,
-                  universal_newlines=False,
-                  stdout=None, stderr=None, env=None):
+        def popen(self, args, cwd, shell=None, stdout=None, stderr=None, env=None, **_):
             pm = pcallMock(args, cwd, env, stdout, stderr, shell)
             self._pcalls.append(pm)
             return pm
@@ -214,28 +236,24 @@ def mocksession(request):
 
 
 @pytest.fixture
-def newmocksession(request):
-    mocksession = request.getfixturevalue("mocksession")
-    newconfig = request.getfixturevalue("newconfig")
-
-    def newmocksession(args, source, plugins=()):
+def newmocksession(mocksession, newconfig):
+    def newmocksession_(args, source, plugins=()):
         mocksession.config = newconfig(args, source, plugins=plugins)
         return mocksession
 
-    return newmocksession
+    return newmocksession_
 
 
 def getdecoded(out):
     try:
         return out.decode("utf-8")
     except UnicodeDecodeError:
-        return "INTERNAL not-utf8-decodeable, truncated string:\n%s" % (
-            py.io.saferepr(out),)
+        return "INTERNAL not-utf8-decodeable, truncated string:\n{}".format(py.io.saferepr(out))
 
 
 @pytest.fixture
-def initproj(request, tmpdir):
-    """Create a factory function for creating example projects
+def initproj(tmpdir):
+    """Create a factory function for creating example projects.
 
     Constructed folder/file hierarchy examples:
 
@@ -257,14 +275,13 @@ def initproj(request, tmpdir):
                 __init__.py
             name.egg-info/       # created later on package build
             setup.py
-
     """
 
-    def initproj(nameversion, filedefs=None, src_root="."):
+    def initproj_(nameversion, filedefs=None, src_root=".", add_missing_setup_py=True):
         if filedefs is None:
             filedefs = {}
         if not src_root:
-            src_root = '.'
+            src_root = "."
         if isinstance(nameversion, six.string_types):
             parts = nameversion.split(str("-"))
             if len(parts) == 1:
@@ -272,44 +289,58 @@ def initproj(request, tmpdir):
             name, version = parts
         else:
             name, version = nameversion
-
         base = tmpdir.join(name)
         src_root_path = _path_join(base, src_root)
-        assert base == src_root_path or src_root_path.relto(base), (
-            '`src_root` must be the constructed project folder or its direct '
-            'or indirect subfolder')
+        assert base == src_root_path or src_root_path.relto(
+            base
+        ), "`src_root` must be the constructed project folder or its direct or indirect subfolder"
 
         base.ensure(dir=1)
         create_files(base, filedefs)
-
-        if not _filedefs_contains(base, filedefs, 'setup.py'):
-            create_files(base, {'setup.py': '''
+        if not _filedefs_contains(base, filedefs, "setup.py") and add_missing_setup_py:
+            create_files(
+                base,
+                {
+                    "setup.py": """
                 from setuptools import setup, find_packages
                 setup(
-                    name='%(name)s',
-                    description='%(name)s project',
-                    version='%(version)s',
+                    name='{name}',
+                    description='{name} project',
+                    version='{version}',
                     license='MIT',
                     platforms=['unix', 'win32'],
-                    packages=find_packages('%(src_root)s'),
-                    package_dir={'':'%(src_root)s'},
+                    packages=find_packages('{src_root}'),
+                    package_dir={{'':'{src_root}'}},
                 )
-            ''' % locals()})
-
+            """.format(
+                        **locals()
+                    )
+                },
+            )
         if not _filedefs_contains(base, filedefs, src_root_path.join(name)):
-            create_files(src_root_path, {
-                name: {'__init__.py': '__version__ = %r' % version}
-            })
-
-        manifestlines = ["include %s" % p.relto(base)
-                         for p in base.visit(lambda x: x.check(file=1))]
+            create_files(
+                src_root_path,
+                {
+                    name: {
+                        "__init__.py": textwrap.dedent(
+                            '''
+                """ module {} """
+                __version__ = {!r}'''
+                        )
+                        .strip()
+                        .format(name, version)
+                    }
+                },
+            )
+        manifestlines = [
+            "include {}".format(p.relto(base)) for p in base.visit(lambda x: x.check(file=1))
+        ]
         create_files(base, {"MANIFEST.in": "\n".join(manifestlines)})
-
-        print("created project in %s" % (base,))
+        print("created project in {}".format(base))
         base.chdir()
         return base
 
-    return initproj
+    return initproj_
 
 
 def _path_parts(path):
@@ -364,3 +395,38 @@ def create_files(base, filedefs):
         elif isinstance(value, six.string_types):
             s = textwrap.dedent(value)
             base.join(key).write(s)
+
+
+@pytest.fixture()
+def mock_venv(monkeypatch):
+    """This creates a mock virtual environment (e.g. will inherit the current interpreter).
+    Note: because we inherit, to keep things sane you must call the py environment and only that;
+    and cannot install any packages. """
+
+    class ProxyCurrentPython:
+        @classmethod
+        def readconfig(cls, path):
+            assert path.dirname.endswith("{}py".format(os.sep))
+            return CreationConfig(
+                md5=getdigest(sys.executable),
+                python=sys.executable,
+                version=tox.__version__,
+                sitepackages=False,
+                usedevelop=False,
+                deps=[],
+                alwayscopy=False,
+            )
+
+    monkeypatch.setattr(CreationConfig, "readconfig", ProxyCurrentPython.readconfig)
+
+    def venv_lookup(venv, name):
+        assert name == "python"
+        return sys.executable
+
+    monkeypatch.setattr(VirtualEnv, "_venv_lookup", venv_lookup)
+
+    @tox.hookimpl
+    def tox_runenvreport(venv, action):
+        return []
+
+    monkeypatch.setattr(venv, "tox_runenvreport", tox_runenvreport)
